package wmo

import (
	"context"
	"go.mongodb.org/mongo-driver/mongo"
	ct "gogs.mikescher.com/BlackForestBytes/goext/cursortoken"
	"gogs.mikescher.com/BlackForestBytes/goext/langext"
	"reflect"
)

type EntityID = any

type fullTypeRef[TData any] struct {
	IsPointer      bool
	Kind           reflect.Kind
	Type           reflect.Type
	UnderlyingType reflect.Type
	Name           string
	Index          []int
}

type Coll[TData any] struct {
	coll        *mongo.Collection
	dataTypeMap map[string]fullTypeRef[TData]
}

func (c *Coll[TData]) Collection() *mongo.Collection {
	return c.coll
}

func (c *Coll[TData]) Name() string {
	return c.coll.Name()
}

func (c *Coll[TData]) Indexes() mongo.IndexView {
	return c.coll.Indexes()
}

func (c *Coll[TData]) Drop(ctx context.Context) error {
	return c.coll.Drop(ctx)
}

func (c *Coll[TData]) createToken(fieldPrimary string, dirPrimary ct.SortDirection, fieldSecondary *string, dirSecondary *ct.SortDirection, lastEntity TData, pageSize *int) (ct.CursorToken, error) {

	valuePrimary, err := c.getFieldValueAsTokenString(lastEntity, fieldPrimary)
	if err != nil {
		return ct.CursorToken{}, err
	}

	valueSeconary := ""
	if fieldSecondary != nil && dirSecondary != nil {
		valueSeconary, err = c.getFieldValueAsTokenString(lastEntity, *fieldSecondary)
		if err != nil {
			return ct.CursorToken{}, err
		}
	}

	return ct.CursorToken{
		Mode:           ct.CTMNormal,
		ValuePrimary:   valuePrimary,
		ValueSecondary: valueSeconary,
		Direction:      dirPrimary,
		PageSize:       langext.Coalesce(pageSize, 0),
		Extra:          ct.Extra{},
	}, nil
}
