package wmo

import (
	"context"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo/options"
)

func (c *Coll[TData]) FindOneAndUpdate(ctx context.Context, filterQuery bson.M, updateQuery bson.M) (TData, error) {
	var res TData

	err := c.coll.FindOneAndUpdate(ctx, filterQuery, updateQuery, options.FindOneAndUpdate().SetReturnDocument(options.After)).Decode(&res)
	if err != nil {
		return *new(TData), err
	}

	return res, nil
}

func (c *Coll[TData]) UpdateOne(ctx context.Context, filterQuery bson.M, updateQuery bson.M) error {
	_, err := c.coll.UpdateOne(ctx, filterQuery, updateQuery)
	if err != nil {
		return err
	}

	return nil
}

func (c *Coll[TData]) UpdateOneByID(ctx context.Context, id EntityID, updateQuery bson.M) error {
	_, err := c.coll.UpdateOne(ctx, bson.M{"_id": id}, updateQuery)
	if err != nil {
		return err
	}

	return nil
}

func (c *Coll[TData]) ReplaceOne(ctx context.Context, id EntityID, value TData) error {
	_, err := c.coll.UpdateOne(ctx, bson.M{"_id": id}, value)
	if err != nil {
		return err
	}

	return nil
}

func (c *Coll[TData]) FindOneAndReplace(ctx context.Context, id EntityID, value TData) (TData, error) {
	var res TData

	err := c.coll.FindOneAndUpdate(ctx, bson.M{"_id": id}, value, options.FindOneAndUpdate().SetReturnDocument(options.After)).Decode(&res)
	if err != nil {
		return *new(TData), err
	}

	return res, nil
}
